@@ -50,10 +50,8 @@ BlockDialog.prototype.show = function() {
     
     this.dialog = WidgetFactory.createDialog(BlockDialog.includeBasePath, "BlockDialog.ui", EAction.getMainWindow());
 
-    var widgets = getWidgets(this.dialog);
-    var leBlockName = widgets["BlockName"];
-    var rx = new RegExp("[^<>/\\\\\":;\?\*|,=`]{1,255}");
-    this.validator = new QRegExpValidator(rx, leBlockName);
+    var leBlockName = this.dialog.findChild("BlockName");
+    this.validator = createValidator("[^<>/\\\\\":;\?\*|,=`]{1,255}", leBlockName);
     leBlockName.setValidator(this.validator);
     leBlockName.textChanged.connect(this, this.validate);
     var creatingBlock = isNull(this.block);
@@ -84,7 +82,7 @@ BlockDialog.prototype.show = function() {
     }
 
     if (!this.dialog.exec()) {
-        destr(this.dialog);
+        destrDialog(this.dialog);
         EAction.activateMainWindow();
         return undefined;
     }
@@ -93,7 +91,7 @@ BlockDialog.prototype.show = function() {
 	
     if (!creatingBlock) {
         // rename block:
-        destr(this.dialog);
+        destrDialog(this.dialog);
         EAction.activateMainWindow();
         this.block.setName(text);
         return this.block;
@@ -101,7 +99,7 @@ BlockDialog.prototype.show = function() {
     else {
         // create new block:
         var block = new RBlock(this.document, text, new RVector(0,0));
-        destr(this.dialog);
+        destrDialog(this.dialog);
         EAction.activateMainWindow();
         return block;
     }
@@ -111,21 +109,19 @@ BlockDialog.prototype.show = function() {
  * Block name validation.
  */
 BlockDialog.prototype.validate = function() {
-    var widgets = getWidgets(this.dialog);
-    var leBlockName = widgets["BlockName"];
+    var leBlockName = this.dialog.findChild("BlockName");
     return BlockDialog.validate(this.block, leBlockName.text, this.document, this.dialog, this.validator, this.allowOverwrite, this.allowSameName);
 };
 
 BlockDialog.validate = function(block, blockName, document, dialog, validator, allowOverwrite, allowSameName) {
     var creatingBlock = isNull(block);
-    var widgets = getWidgets(dialog);
 
-    var leBlockName = widgets["BlockName"];
+    var leBlockName = dialog.findChild("BlockName");
     if (isNull(leBlockName)) {
         return false;
     }
 
-    var message = widgets["Message"];
+    var message = dialog.findChild("Message");
     var pos = 0;
     var acceptable = true;
     message.clear();
@@ -163,8 +159,9 @@ BlockDialog.validate = function(block, blockName, document, dialog, validator, a
         ret = false;
     }
 
-    if (!isNull(widgets["ButtonBox"])) {
-        widgets["ButtonBox"].button(QDialogButtonBox.Ok).enabled = acceptable;
+    var buttonBox = dialog.findChild("ButtonBox");
+    if (!isNull(buttonBox)) {
+        buttonBox.button(QDialogButtonBox.Ok).enabled = acceptable;
     }
 
     return ret;
