@@ -235,6 +235,7 @@ SvgHandler.prototype.startElement = function(namespaceURI, localName, qName, att
         break;
 
     case "g":
+        this.svgImporter.importGroup(atts.value("id"));
         break;
 
     case "path":
@@ -308,6 +309,10 @@ SvgHandler.prototype.endElement = function(namespaceURI, localName, qName) {
     this.displayStack.pop();
     this.elementStack.pop();
 
+    if (localName==="g") {
+        this.svgImporter.groupEnd();
+    }
+
     return true;
 };
 
@@ -327,6 +332,11 @@ function SvgImporter(document, resolution) {
         resolution = 72;
     }
 
+    if (isNull(document)) {
+        RFileImporterAdapter.call(this);
+        return;
+    }
+
     RFileImporterAdapter.call(this, document);
     this.setDocument(document);
     this.transform = new QTransform();
@@ -382,7 +392,7 @@ SvgImporter.prototype.importFile = function(fileName) {
 };
 
 SvgImporter.prototype.importShape = function(shape) {
-    var shapeP;
+    //var shapeP;
 
     if (isArcShape(shape) || isEllipseShape(shape) || isCircleShape(shape)) {
         var t = this.transform;
@@ -393,13 +403,13 @@ SvgImporter.prototype.importShape = function(shape) {
         );
     }
     else {
-        shapeP = shape.getTransformed(this.transform);
-        if (isFunction(shapeP.data)) {
-            shape = shapeP.data();
-        }
-        else {
-            shape = shapeP;
-        }
+        shape = shape.getTransformed(this.transform);
+        // if (isFunction(shapeP.data)) {
+        //     shape = shapeP.data();
+        // }
+        // else {
+        //     shape = shapeP;
+        // }
     }
 
     shape.scale(new RVector(1.0, -1.0));
@@ -427,9 +437,23 @@ SvgImporter.prototype.importShape = function(shape) {
     if (!isNull(this.style.stroke)) {
         entity.setColor(new RColor(this.style.stroke));
     }
+
+    this.preImportObject(entity);
+
     this.importObject(entity);
 };
 
+SvgImporter.prototype.preImportObject = function(obj) {
+};
+
+SvgImporter.prototype.importGroup = function(name) {
+    // ignore groups
+};
+
+SvgImporter.prototype.groupEnd = function() {
+    // ignore groups
+};
+
 SvgImporter.prototype.importCircle = function(x, y, r) {
     this.importShape(new RCircle(new RVector(x, y), r));
 };
@@ -860,7 +884,7 @@ SvgImporter.prototype.importPath = function(dData) {
         var shape = shapes[i];
 
         if (isLineShape(shape) || isArcShape(shape)) {
-            pl.appendShape(shape);
+            pl.appendShape(getPtr(shape));
         }
         else {
             // shape cannot be part of a polyline or is undefined (gap):
