@@ -76,6 +76,13 @@ function requiresFramework(minVersion, maxVersion) {
     }
 }
 
+function checkMinVersion(minVersion) {
+    var frameworkV = getVersionNumber(RSettings.getMajorVersion(), RSettings.getMinorVersion(), RSettings.getRevisionVersion(), RSettings.getBuildVersion());
+    var minV = getVersionNumber(minVersion[0], minVersion[1], minVersion[2], minVersion[3]);
+
+    return frameworkV >= minV;
+}
+
 function getVersionNumber(major, minor, revision, build) {
     return major*1e9 + minor*1e6 + revision*1e3 + build;
 }
@@ -141,11 +148,24 @@ function destr(obj) {
     }
 }
 
+function destrDialog(dlg) {
+    if (RSettings.getQtVersion() >= 0x060000) {
+        // delete later in Qt 6
+        // immediate delete might cause unpredictable crashes in event processing:
+        dlg.deleteLater();
+    }
+    else {
+        if (isFunction(dlg.destroy)) {
+            dlg.destroy();
+        }
+    }
+}
+
 /**
  * Get pointer from shared pointer object using data() (Qt5) or implicitly (Qt6).
  */
 function getPtr(p) {
-    if (RSettings.getQtVersion() > 0x060000) {
+    if (RSettings.getQtVersion() >= 0x060000) {
         return p;
     }
     else {
@@ -161,7 +181,7 @@ function getPtr(p) {
 /**
  * Checks if the given object is undefined or null.
  *
- * \return true if the given object is defined and not null.
+ * \return false if the given object is defined and not null.
  * This method also supports shared pointers, i.e. obj.isNull()==true.
  */
 function isNull(obj) {
@@ -169,8 +189,17 @@ function isNull(obj) {
         return true;
     }
 
+    if (obj==null) {
+        return true;
+    }
+
+    if (RSettings.getQtVersion() >= 0x060000) {
+        if (obj.hasOwnProperty("__PROXY__")) {
+            return isNull(obj.__PROXY__);
+        }
+    }
+
     return (
-        obj==null ||
         // wrapped object is NULL:
         (typeof(obj.isNullWrapper)==="function" && obj.isNullWrapper()===true) ||
         // shared pointer is NULL:
@@ -277,6 +306,15 @@ function isQWidget(obj) {
     return (isQObject(obj) && typeof(obj.show)==="function");
 }
 
+/**
+ * Checks if the given object is a valid QAction.
+ *
+ * \return true if the given object is a QAction.
+ */
+function isQAction(obj) {
+    return (isQObject(obj) && typeof(obj.actionGroup)==="function");
+}
+
 /**
  * \return true if the given object is a separator (a frame of type vertical line).
  */
@@ -681,6 +719,18 @@ function isPolylineEntity(obj) {
     }
 }
 
+/**
+ * \return true if the given object is a wipeout entity (RWipeoutEntity).
+ */
+function isWipeoutEntity(obj) {
+    if (RSettings.getQtVersion() < 0x060000) {
+        return isOfType(obj, RWipeoutEntity) || isOfType(obj, RWipeoutEntityPointer);
+    }
+    else {
+        return isOfType(obj, RWipeoutEntity);
+    }
+}
+
 /**
  * \return true if the given object is a geometrically closed polyline entity.
  */
@@ -1226,6 +1276,10 @@ function entityTypeToClass(type) {
         return "RTextEntity";
     case RS.EntityViewport:
         return "RViewportEntity";
+    case RS.EntityWipeout:
+        return "RWipeoutEntity";
+    case RS.EntityXRef:
+        return "RXRefEntity";
     default:
         return undefined;
     }
@@ -1286,9 +1340,9 @@ function entityTypeToString(type, plural) {
     case RS.EntityDimAngular:
         return plural ? qsTr("Angular Dimensions") : qsTr("Angular Dimension");
     case RS.EntityDimAngular2L:
-        return plural ? qsTr("Angular Dimensions (2 Line)") : qsTr("Angular Dimension (2 Line)");
+        return plural ? qsTr("Angular Dimensions (2 Lines)") : qsTr("Angular Dimension (2 Lines)");
     case RS.EntityDimAngular3P:
-        return plural ? qsTr("Angular Dimensions (3 Point)") : qsTr("Angular Dimension (3 Point)");
+        return plural ? qsTr("Angular Dimensions (3 Points)") : qsTr("Angular Dimension (3 Points)");
     case RS.EntityDimArcLength:
         return plural ? qsTr("Arc Dimension") : qsTr("Arc Dimension");
     case RS.EntityDimDiametric:
@@ -1333,8 +1387,20 @@ function entityTypeToString(type, plural) {
         return plural ? qsTr("Texts") : qsTr("Text");
     case RS.EntityViewport:
         return plural ? qsTr("Viewports") : qsTr("Viewport");
+    case RS.EntityWipeout:
+        return plural ? qsTr("Wipeouts") : qsTr("Wipeout");
+    case RS.EntityXRef:
+        return plural ? qsTr("External References") : qsTr("External Reference");
     case RS.EntityUnknown:
+        return plural ? qsTr("Unknown Entities") : qsTr("Unknown Entity");
     default:
+        if (type>=RS.EntityCustom && type<RS.EntityUser) {
+            var handler = RCustomEntityRegistry.getHandler(type);
+            if (!isNull(handler)) {
+                var name = handler.getEntityName(plural);
+                return qsTranslate("RCustomEntity", name);
+            }
+        }
         return plural ? qsTr("Unknown Entities") : qsTr("Unknown Entity");
     }
 }
@@ -1430,6 +1496,10 @@ function getEntityType(typeName) {
         return RS.EntityText;
     case "Viewport":
         return RS.EntityViewport;
+    case "Wipeout":
+        return RS.EntityWipeout;
+    case "External Reference":
+        return RS.EntityXRef;
     default:
         return RS.ObjectUnknown;
     }
@@ -1491,7 +1561,19 @@ function objectFromPath(path) {
  */
 function getWidgetPath(widget) {
     var pw = widget.parentWidget();
-    var str = widget.objectName !== "" ? widget.objectName : widget.toString();
+    var str = "";
+    if (RSettings.getQtVersion()>=0x060000) {
+        if (widget.objectName !== "") {
+            str = widget.objectName;
+        }
+        else {
+            return getWidgetPath(pw);
+        }
+    }
+    else {
+        str = widget.objectName !== "" ? widget.objectName : widget.toString();
+    }
+
     if (isNull(pw)) {
         return str;
     }
@@ -1875,6 +1957,12 @@ Array.prototype.remove = function(val, compareFunction) {
     }
 };
 
+Array.prototype.removeAt = function(index) {
+    var part1 = this.slice(0, index);
+    var part2 = this.slice(index + 1);
+    return (part1.concat(part2));
+};
+
 /**
  * Removes all matches of the given value from the array.
  * \param The value(s) to remove.
@@ -1933,7 +2021,7 @@ Array.prototype.clone = function() {
  */
 Array.prototype.sortNumerical = function() {
     Array.prototype.sort.call(this, function(a,b) { return a - b });
-}
+};
 
 Array.alphaNumericalSorter = function(a, b) {
     return RS.compareAlphanumerical(a, b);
@@ -2498,43 +2586,37 @@ function stringToDirectDistanceEntry(relativeZero, cursorPosition, str) {
  * Creates and returns a new entity based on the given shape.
  */
 function shapeToEntity(document, shape) {
-    var s = shape;
-    if (isFunction(s.data)) {
-        s = s.data();
+    if (isNull(shape)) {
+        return undefined;
     }
 
-    if (isPointShape(s)) {
+    var s = getPtr(shape);
+
+    switch (s.getShapeType()) {
+    case RShape.Point:
         return new RPointEntity(document, new RPointData(s.getPosition()));
-    }
-    else if (isLineShape(s)) {
+    case RShape.Line:
         return new RLineEntity(document, new RLineData(s));
-    }
-    else if (isRayShape(s)) {
+    case RShape.Ray:
         return new RRayEntity(document, new RRayData(s));
-    }
-    else if (isXLineShape(s)) {
+    case RShape.XLine:
         return new RXLineEntity(document, new RXLineData(s));
-    }
-    else if (isArcShape(s)) {
+    case RShape.Arc:
         return new RArcEntity(document, new RArcData(s));
-    }
-    else if (isCircleShape(s)) {
+    case RShape.Circle:
         return new RCircleEntity(document, new RCircleData(s));
-    }
-    else if (isEllipseShape(s)) {
+    case RShape.Ellipse:
         return new REllipseEntity(document, new REllipseData(s));
-    }
-    else if (isPolylineShape(s)) {
+    case RShape.Polyline:
         return new RPolylineEntity(document, new RPolylineData(s));
-    }
-    else if (isSplineShape(s)) {
+    case RShape.Spline:
         return new RSplineEntity(document, new RSplineData(s));
-    }
-    else if (isTriangleShape(s)) {
+    case RShape.Triangle:
         return new RSolidEntity(document, new RSolidData(s));
+    default:
+        return undefined;
     }
 
-    qWarning("shapeToEntity: unknown shape: ", s);
     return undefined;
 }
 
@@ -2589,21 +2671,15 @@ function modifyEntity(op, entity, shape) {
         (isCircleEntity(entity) && isArcShape(shape))) {
 
         var e = shapeToEntity(entity.getDocument(), shape);
-//        if (isFunction(entity.data)) {
-//            e.copyAttributesFrom(entity.data());
-//            op.deleteObject(entity.data());
-//        }
-//        else {
-            e.copyAttributesFrom(entity);
-            op.deleteObject(entity);
-//        }
+        e.copyAttributesFrom(getPtr(entity));
+        op.deleteObject(entity);
 
         op.addObject(e, false);
         return true;
     }
 
     if (isFunction(entity.setShape)) {
-        entity.setShape(shape);
+        entity.setShape(getPtr(shape));
         op.addObject(entity, false);
         return true;
     }
@@ -2801,22 +2877,15 @@ function initUserShortcuts() {
             continue;
         }
 
-        var match = action.getScriptFile().contains("OpenFile");
-
         var scStringList = RSettings.getValue("Shortcuts/" + key);
 
-        if (match) qDebug("scStringList", scStringList);
-
         // explicitly no shortcuts:
         if (isNull(scStringList)) {
-            if (match) qDebug("explicitly no shortcuts");
             action.setShortcuts([]);
             action.removeShortcuts();
             continue;
         }
 
-        if (match) qDebug("setting shortcuts to ", scStringList);
-
         //action.setShortcutsFromStrings(scStringList);
         //action.setShortcuts(scStringList);
 
@@ -2918,42 +2987,6 @@ function addActionsToWidgets() {
             }
             var w = appWin.findChild(wn);
             if (!isNull(w)) {
-
-                // workaround for QTBUG-38256 (action not triggered for letter based shortcuts in SUB menus):
-                // workaround for QTBUG-57990 (action not triggered for letter based shortcuts in ALL menus):
-                /*
-                if ((RSettings.getQtVersion()<0x050500 && RSettings.getQtVersion()>=0x050000) ||
-                    (RSettings.getQtVersion()===0x050800 && RS.getSystemId()==="linux")) {
-
-                    if (k===0) {
-                        var shortcuts = a.shortcuts();
-
-                        var scObjs = appWin.property("DelegatedShortcutsObjs");
-                        if (isNull(scObjs)) {
-                            scObjs=[];
-                        }
-                        for (var si=0; si<shortcuts.length; si++) {
-                            var scObj = new QShortcut(shortcuts[si], a.parentWidget(), 0, 0, Qt.WindowShortcut)
-                            scObj.activated.connect(a, "trigger");
-                            scObjs.push(scObj);
-                        }
-                        appWin.setProperty("DelegatedShortcutsObjs", scObjs);
-
-                        var sc = a.shortcut.toString();
-                        if (sc.length>0) {
-                            if (RS.getSystemId()==="linux") {
-                                a.setShortcutText("[" + sc + "]");
-                            }
-                            else {
-                                a.setShortcutText(" " + sc);
-                            }
-                        }
-                        a.setProperty("DelegatedShortcuts", shortcuts);
-                        a.setDefaultShortcuts([]);
-                    }
-                }
-                */
-
                 if (visibility) {
                     RGuiAction.addToWidget(a, w);
                     addedToWidget = true;
@@ -3019,8 +3052,12 @@ function isUrl(urlString) {
 }
 
 function getDontUseNativeDialog() {
-    // don't use KDE file dialog (workaround for file type filter bug):
-    return RS.getWindowManagerId()==="kde" || RSettings.getBoolValue("SaveAs/UseSystemFileDialog", true)===false;
+    if (RS.getWindowManagerId()==="kde") {
+        // don't use KDE file dialog (workaround for file type filter bug):
+        return true;
+    }
+
+    return RSettings.getBoolValue("SaveAs/UseSystemFileDialog", true)===false;
 };
 
 function autoPath(path) {
@@ -3484,6 +3521,15 @@ function openFiles(args, createNew, close) {
     }
 }
 
+function createValidator(rx, parent) {
+    if (RSettings.getQtVersion()>=0x060000) {
+        return new QRegularExpressionValidator(new QRegularExpression(rx), parent);
+    }
+    else {
+        return new QRegExpValidator(new RegExp(rx), parent);
+    }
+}
+
 // fix QPlainTextEdit API for Qt 5:
 if (!isFunction(QPlainTextEdit.prototype.toPlainText)) {
     QPlainTextEdit.prototype.toPlainText = function() {
